/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.implementations;

import java.io.*;
import java.util.*;

import talon.*;
import talon.components.*;

import org.apache.log4j.*;
import org.apache.log4j.xml.*;
import org.apache.log4j.helpers.Loader;

/**
 * Simple logging abstraction for Log4J
 *
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
 */
public class Log4JLogger extends BaseComponent implements Component, Logger {

    private Category root = Category.getRoot();
    
    /**
     * Default constructor when used as a Component
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public Log4JLogger() {

        //init Log4J

        String configuration = System.getProperty( "log4j.configuration" );

        Assert.assert( configuration, "Configuration not specified.  Need to set the system param: 'log4j.configuration'");

        if ( configuration.startsWith( "file:/" ) ) {
        
            DOMConfigurator.configure( configuration );
            
        } else {

                    
            DOMConfigurator.configure( Loader.getResource( configuration,
                                                           Log4JLogger.class ) );

        }

    }

    /**
     * Allow a new Log4JLogger to be created with a specific category...
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public Log4JLogger( Category category ) {

        this.root = category;
        
    }
    
    /**
     * Start with a specific project.
     *
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public Log4JLogger( ComponentHandle handle,
                        String project ) {

        this();
        this.setComponentHandle( handle );
        
        root = Category.getInstance( project );

        debug( "The following appenders were found...", this ); 
        
        Enumeration cats = root.getAllAppenders();
        
        while ( cats.hasMoreElements() ) {

            Appender next = (Appender)cats.nextElement();

            debug( "Log4J using the following appender: " + next + getClass().toString(), this );

        } 
        
    }
    
    /**
     * Perform any necessary initialization
     *
     * @see Component
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void init() throws TalonException { }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void message( Object message ) {

        root.info( message );
        
    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void message( Object message, Object category ) {

        Category.getInstance( category.getClass() ).info( message );
        
    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void debug( Object message ) {

        root.debug( message );
        
    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void debug( Object message, Object category ) {

        Category.getInstance( category.getClass() ).debug( message );
        
    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void warn( Object message ) {

        root.warn( message );
        
    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void warn( Object message, Object category ) {

        Category.getInstance( category.getClass() ).warn( message );

    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message ) {

        root.error( message );
        
    }


    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message, Throwable throwable ) {

        root.error( message, throwable );
        
    }

    
    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public void error( Object message, Object category, Throwable t ) {

        Category.getInstance( category.getClass() ).error( message, t );

    }


    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public Logger getLogger( Class category ) {

        Logger logger = new Log4JLogger( Category.getInstance( category ) );

        logger.setComponentHandle( this.getComponentHandle() );
        
        return logger;

    }

    /**
     * @see Logger
     * @author <a href="mailto:burton@relativity.yi.org">burtonator</a>
     * @version $Id: Log4JLogger.java,v 1.4 2001/08/02 07:13:03 burton Exp $
     */
    public Logger getLogger( Object category ) {

        return getLogger( category.getClass() );
        
    }
    
}
