/*
 * Copyright 2001 OpenPrivacy.org.  All rights reserved.
 *
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the LICENSE which you should have obtaind with this package. 
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.
 */

package talon.implementations;

import java.io.*;
import java.net.*;
import java.util.*;

import talon.*;
import talon.components.*;
import talon.interfaces.*;
import talon.resources.*;
import talon.util.*;
import talon.util.net.*;


/**
 * 
 * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
 * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
 */
public class TalonPropertyManager extends FilePropertyManager {

    public static final String[] REQUIRED_PROPERTIES = { "project" };
    
    public static final String INIT_PROPERTY_PROJECT = "project";

    private String propertyFilename = null;
    
    /**
     * Constructor when used as a Component
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    public TalonPropertyManager( ) { } 
    
    /**
     * Constructor when used as an Object
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    public TalonPropertyManager( ComponentHandle handle,
                                 String project ) throws TalonException {
        
        this.setComponentHandle( handle );
        handle.getInitProperties().setProperty( "project", project );
        this.init();

    } 

    /**
     * Perform any necessary initialization
     *
     * @see Component
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    public void init() throws TalonException {

        String resource = System.getProperty( TalonResources.TALON_PROPERTIES_KEY );

        if ( resource != null ) {
            
            //here the developer has choosen to override "project".properties
            //detection and provide their own resource.

            System.out.println( "Loading Talon resource: " + resource );
            
            try {
                
                Properties props = new Properties();

                ResourceMapper rm = TalonResources.getInstance().getResourceMapper();

                props.load( rm.getResource( resource ).getURL().openStream() );

                this.setProperties( props );

                this.propertyFilename = resource;
                
                return;
                
            } catch ( IOException e ) {

                throw new TalonException( e );

            }
            
        } 
        
        
        //project isn't required if we loaded this from a resource as above.
        getComponentHandle().getInitProperties().require( REQUIRED_PROPERTIES );
        
        String project = this.getComponentHandle().getInitProperties().getString( INIT_PROPERTY_PROJECT );

        String[] locations = getFileLocations( project );
        
        String file = null;
        
        for( int i = 0; i < locations.length; ++i ) {

            if ( new File( locations[i] ).exists() ) {
                file = locations[i];
            } 
                        
        }

        //if the .properties file can not be found... throw an Exception
        if ( file == null ) {
            throw new TalonException( "Unable to find a property file to use (file was null)" );
        } 
        
        //keep track of the file this was initialized from.
        this.propertyFilename = file;
        
        try {

            Properties props = new Properties();
            props.load( new FileInputStream( file ) );

            this.setProperties( props );
            
        } catch ( Exception e ) {

            e.printStackTrace();

            throw new TalonException( e.getMessage() );

        }
           
    }

    /**
     * Given a project name, determine where its files might be stored.
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    private String[] getFileLocations( String project ) {

        
        Vector v = new Vector();

        //if there is a System Property defined named PROJECT_PROPERTIES... use that

        String prop = project.toUpperCase() + "_PROPERTIES";
        String value = System.getProperty( prop );
        
        if ( value != null ) {
            v.addElement( value );
        } 
        
        v.addElement( System.getProperty( "user.home" ) +
                      File.separator +
                      project +
                      ".properties" );

        v.addElement( System.getProperty( "java.home" ) +
                      File.separator +
                      project +
                      ".properties" );

        v.addElement( "/etc/" +
                      project +
                      ".properties" );


        String[] s = new String[v.size()];
        v.copyInto(s);

        return s;
        
    }

    /**
     * Get the property file that this was initialized from. 
     *
     * @author <a href="mailto:burton@relativity.yi.org">Kevin A. Burton</a>
     * @version $Id: TalonPropertyManager.java,v 1.5 2001/12/02 05:37:48 burton Exp $
     */
    public String getPropertyFilename() {
        return this.propertyFilename;
    }
    
}
